#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
@author: P.Cizek
"""
import matplotlib.pyplot as plt

from ompl import base as ob
from ompl import geometric as og

############################
# Plotting functions
############################
def plot_points(points, specs = 'r'):
	x_val = [x[0] for x in points]
	y_val = [x[1] for x in points]
	plt.plot(x_val, y_val, specs)
	plt.draw()

def plot_edges(edges, specs = 'r'):
	for tup in edges:
		x_val = [x[0] for x in tup]
		y_val = [x[1] for x in tup] 
		plt.plot(x_val, y_val, specs)
	plt.draw()

############################
# OMPL planning
############################
def isStateValid(state):
	#method for collision detection checking
	x = state[0]
	y = state[1]
	return (0.5*(x - 5)**2 + 2) > y

def plan(start, goal):
	stateSpace = ob.RealVectorStateSpace()  #set state space
	stateSpace.addDimension(0.0, 10.0) #set width
	stateSpace.addDimension(0.0, 10.0) #set height
	
	#create a simple setup object
	task = og.SimpleSetup(stateSpace)
	
	#set methods for collision detection and resolution of the checking 
	task.setStateValidityChecker(ob.StateValidityCheckerFn(isStateValid))
	task.getSpaceInformation().setStateValidityCheckingResolution(0.001)
	
	#setting start and goal positions
	start_pose = ob.State(task.getStateSpace())   
	goal_pose = ob.State(task.getStateSpace())
	start_pose()[0] = start[0]
	start_pose()[1] = start[1]
	goal_pose()[0] = goal[0]
	goal_pose()[1] = goal[1]
	task.setStartAndGoalStates(start_pose, goal_pose)
	
	#setting particular planner from the supported planners
	info = task.getSpaceInformation()
	planner = og.FMT(info)   # RRT, RRTConnect, FMT, ...
	task.setPlanner(planner)
	
	#find the solution
	solution = task.solve()
	
	#simplify the solution
	if solution:
		task.simplifySolution()
	
	#retrieve found path
	plan = task.getSolutionPath()
	
	#extract path and plot it
	path = []
	for i in range(plan.getStateCount()):
		path.append((plan.getState(i)[0], plan.getState(i)[1]))
	
	print(len(path))
	plot_points(path,'r')
	
	#retrieve planner data
	plannerdata = ob.PlannerData(info)
	planner.getPlannerData(plannerdata)
	
	#getting vertices
	nv = plannerdata.numVertices()
	print(nv)
	g = []
	for i in range(0, nv):
		vert = plannerdata.getVertex(i)
		g.append((vert.getState()[0], vert.getState()[1]))
	
	#getting edges
	e = []
	for i in range(0, nv):
		for j in range(0, nv):
			if plannerdata.edgeExists(i, j):
				e.append((g[i], g[j]))
	
	plot_edges(e,'g')
	plt.show()

############################
# Main method
############################
if __name__ == "__main__":
	plan([0, 10], [10, 10])